/**
 * src/qchaco/ChacoCmd.cpp
 * written by Sven Oliver Moll
 * 
 * distributed under the terms of the GNU Lesser General Public License (LGPL)
 * available at http://www.gnu.org/licenses/lgpl.html
 */

/* class declaration */
#include "ChacoCmd.hpp"

/* system headers */
#include <stdio.h>

/* Qt headers */
#include <QFileInfo>
#include <QProcess>
#include <QSettings>
#include <QTime>
#include <QTemporaryFile>

/* local library headers */

/* local headers */


ChacoCmd::ChacoCmd( QObject *parent )
: QThread( parent )
, mErrored( false )
, mpProcess( new QProcess( this ) )
, mFilePath( "chacocmd" )
{
   connect( mpProcess, SIGNAL(readyRead()),
            this, SLOT(handleReadyRead()) );
   moveToThread( this );
   start();
}


ChacoCmd::~ChacoCmd()
{
}


void ChacoCmd::run()
{
   setTerminationEnabled( true );
   exec();
}


void ChacoCmd::resetError()
{
   mErrored = false;
}


void ChacoCmd::setFilePath( const QString &name )
{
   mFilePath = name;
}


const QString &ChacoCmd::filePath()
{
   return mFilePath;
}


void ChacoCmd::startCore( int slot )
{
   QStringList args;
   args << "--start" << QString::number( slot );
   sendCmd( args );
}


void ChacoCmd::flashRbf( int slot, const QString &coreName,
                         const QByteArray &romData )
{
   QFileInfo fileInfo( coreName );
   QStringList args;
   args << "--flashrbf" << QString::number( slot )
         << coreName;

   if( !romData.isEmpty() )
   {
      QTemporaryFile dataFile;
      dataFile.open();
      dataFile.write( romData );
      dataFile.close();

      args << dataFile.fileName();
      sendCmd( args, fileInfo.size() + romData.size() );
   }
   else
   {
      sendCmd( args, fileInfo.size() );
   }
}


void ChacoCmd::upload( unsigned long addr, const QByteArray &data )
{
   QTemporaryFile dataFile;
   dataFile.open();
   dataFile.write( data );
   dataFile.close();

   QStringList args;
   args << "--addr" << "0x" + QString::number( addr, 16 )
        << "--writemem" << dataFile.fileName();
   sendCmd( args, data.size() );
}


void ChacoCmd::upload( unsigned long addr, const QString &fileName )
{
   QStringList args;
   args << "--addr" << "0x" + QString::number( addr, 16 )
        << "--writemem" << fileName;
   sendCmd( args, QFileInfo( fileName ).size() );
}


void ChacoCmd::download( unsigned long addr, unsigned long size,
                         QObject *target, const QString &method )
{
   QByteArray data;

   QTemporaryFile dataFile;
   dataFile.open();
   dataFile.close();

   QStringList args;
   args << "--addr" << "0x" + QString::number( addr, 16 )
        << "--len"  << "0x" + QString::number( size, 16 )
        << "--readmem" << dataFile.fileName();
   sendCmd( args, size );

   QFile readFile( dataFile.fileName() );
   if( readFile.open( QIODevice::ReadOnly ) )
   {
      data = readFile.readAll();
      readFile.close();
   }

   if( !QMetaObject::invokeMethod( target, method.toAscii().constData(), Qt::QueuedConnection,
                                   Q_ARG( const QByteArray &, data ) ) )
   {
      qFatal( "%s:%d call failed in %s", __FILE__, __LINE__, Q_FUNC_INFO );
   }
}


void ChacoCmd::sendCmd( const QStringList &args, int dataSize )
{
   if( mErrored || mFilePath.isEmpty() )
   {
      return;
   }
   emit output( QString("%1 %2").arg( mFilePath, args.join(" ") ) );
   QTime timer;
   timer.start();
   emit active( true );
   mpProcess->start( mFilePath, args, QIODevice::ReadOnly );
   if( !mpProcess->waitForStarted(5000) )
   {
      mErrored = true;
      emit error( 0x20000000 );
      emit active( false );
      return;
   }
   mpProcess->waitForFinished( -1 );
   emit active( false );
   int elapsed = timer.elapsed();
   int retval  = mpProcess->exitCode() |
                 (mpProcess->exitStatus() != QProcess::NormalExit ) ? 0x10000000 : 0;
   if( retval )
   {
      mErrored = true;
      emit error( retval );
      return;
   }

   if( dataSize )
   {
      emit output( QString("%1 ms, %2 bytes/s").arg(QString::number(elapsed),
                                                    QString::number((dataSize * 1000) / elapsed)) );
   }
   else
   {
      emit output( QString("%1 ms").arg(elapsed) );
   }
}


void ChacoCmd::handleReadyRead()
{
   QString data( QString::fromLocal8Bit( mpProcess->readAll().constData() ) );
   foreach( const QString &line, data.split( "\n", QString::QString::SkipEmptyParts ) )
   {
      emit output( line );
   }
}
