/**
 * src/qchaco/D64File.cpp
 * written by Sven Oliver Moll
 * 
 * distributed under the terms of the GNU Lesser General Public License (LGPL)
 * available at http://www.gnu.org/licenses/lgpl.html
 */

/* class declaration */
#include "D64File.hpp"

/* system headers */

/* Qt headers */
#include <QFile>

/* local library headers */

/* local headers */


#include <QtDebug>

D64File::D64File( const QString &fileName )
: mIsValid( false )
, mZeroFilledBlocks( true )
, m40Tracks( false )
, mpGcrData( ::gcr_create_image() )
, mFileData()
{
   QFile file( fileName );
   file.open( QIODevice::ReadOnly );
   mFileData = file.readAll();
   file.close();
   switch( mFileData.size() )
   {
   case 768*256:
      m40Tracks = true;
      /* slip through */
   case 683*256:
      mIsValid = true;
      break;
   default:
      break;
   }
}


D64File::D64File( int tracks )
: mIsValid( false )
, mZeroFilledBlocks( true )
, m40Tracks( false )
, mpGcrData( ::gcr_create_image() )
, mFileData()
{
   int sectors = 683;
   switch( tracks )
   {
   case 40:
      sectors = 768;
      m40Tracks = true;
      /* slip through */
   case 35:
      mIsValid = true;
      break;
   default:
      break;
   }
   if( mIsValid )
   {
      mFileData.resize( sectors * 256 );
   }
}


D64File::~D64File()
{
   ::gcr_destroy_image( mpGcrData );
}


bool D64File::isValid()
{
   return mIsValid;
}


int D64File::convertBlockNumberTo( int block, enum convertTarget to )
{
   if( block < 357 )
   {
      switch(to)
      {
      case ToTrackNumber:
         return 1 + (block / 21);
      case ToSectorNumber:
         return block % 21;
      case ToSectorsPerTrack:
         return 21;
      case ToGcrSizeOfTrack:
         return 7696;
      case ToSectorOffset:
         return 7696 * (block % 21) / 21;
      default:
         return -1;
      }
   }
   block -= 357;
   if( block < 133 )
   {
      switch(to)
      {
      case ToTrackNumber:
         return 18 + (block / 19);
      case ToSectorNumber:
         return block % 19;
      case ToSectorsPerTrack:
         return 19;
      case ToGcrSizeOfTrack:
         return 7144;
      case ToSectorOffset:
         return 7144 * (block % 19) / 19;
      default:
         return -1;
      }
   }
   block -= 133;
   if( block < 108 )
   {
      switch(to)
      {
      case ToTrackNumber:
         return 25 + (block / 18);
      case ToSectorNumber:
         return block % 18;
      case ToSectorsPerTrack:
         return 18;
      case ToGcrSizeOfTrack:
         return 6672;
      case ToSectorOffset:
         return 6672 * (block % 18) / 18;
      default:
         return -1;
      }
   }
   block -= 108;
   switch(to)
   {
   case ToTrackNumber:
      return 31 + (block / 17);
   case ToSectorNumber:
      return block % 17;
   case ToSectorsPerTrack:
      return 17;
   case ToGcrSizeOfTrack:
      return 6256;
   case ToSectorOffset:
      return 6256 * (block % 17) / 17;
   default:
      return -1;
   }
}


void D64File::formatGcr( char id1, char id2 )
{
   int track;
   int sector;
   int sectors;
   int offset;
   int bytesPerTrack;
   QByteArray payload( 260, mZeroFilledBlocks ? 0 : 1 );
   payload[0] = 0x07;
   if( !mZeroFilledBlocks )
   {
      payload[1]   = 0x4b;
      payload[258] = 0x00;
      payload[259] = 0x00;
   }
   else
   {
      payload[3] = 'E';
      payload[4] = 'M';
      payload[5] = 'P';
      payload[6] = 'T';
      payload[7] = 'Y';
   }
   char checksum = 0;
   for( int i = 0; i < 256; i++ )
   {
      checksum ^= payload.at(i + 1);
   }
   payload[257] = checksum;

   ::memset( &(mpGcrData->data[0]), 0x55, MAX_GCR_TRACKS * NUM_MAX_BYTES_TRACK );
   for( track = 0; track < MAX_GCR_TRACKS; track++ )
   {
      mpGcrData->track_size[track] = NUM_MAX_BYTES_TRACK;
   }
   for( track = 1; track <= MAX_GCR_TRACKS; track++ )
   {
      if( track < 18 )
      {
         bytesPerTrack = 7696;
         sectors = 21;
      }
      else if( track < 25 )
      {
         bytesPerTrack = 7144;
         sectors = 19;
      }
      else if( track < 31 )
      {
         bytesPerTrack = 6672;
         sectors = 18;
      }
      else
      {
         bytesPerTrack = 6256;
         sectors = 17;
      }
      //mpGcrData->track_size[track - 1] = bytesPerTrack;
      for( sector = 0; sector < sectors; sector++ )
      {
         offset = ((track - 1) * NUM_MAX_BYTES_TRACK)
                + ((bytesPerTrack * sector) / sectors);
         ::gcr_convert_sector_to_GCR( (unsigned char*)payload.constData(),
                                      (unsigned char*)&(mpGcrData->data[offset]),
                                      track, sector,
                                      id1, id2, 0 );
      }
   }
   mIsValid = true;
}


void D64File::convertToGcr()
{
   int track;
   int sector;
   int block;
   int offset;
   unsigned char *dataptr;
   for( block = 0; block < (m40Tracks ? 768 : 683); block++ )
   {
      track  = convertBlockNumberTo( block, ToTrackNumber );
      sector = convertBlockNumberTo( block, ToSectorNumber );
      offset = ((track - 1) * NUM_MAX_BYTES_TRACK)
             + convertBlockNumberTo( block, ToSectorOffset );
#if 0
qDebug() << block << track << sector << hex << offset;
      if( (track == 17) && (sector == 20) )
      {
         qDebug() << "break here...";
      }
#endif
      dataptr = (unsigned char*)mFileData.data();
      if( ::gcr_write_sector((unsigned char*)&(mpGcrData->data[offset]),
                             NUM_MAX_BYTES_TRACK, &(dataptr[block * 256]),
                             track, sector) < 0 )
      {
         qWarning( "gcr_write_sector(%d,%d) failed", track, sector );
      }
   }
}


void D64File::convertFromGcr()
{
   int track;
   int sector;
   int block;
   int offset;

   mFileData.fill('x');
   unsigned char *fileData = (unsigned char*)mFileData.data();

   mFileData.resize( (m40Tracks ? 768 : 683) * 256 );
   for( block = 0; block < (m40Tracks ? 768 : 683); block++ )
   {
      track  = convertBlockNumberTo( block, ToTrackNumber );
      sector = convertBlockNumberTo( block, ToSectorNumber );
      offset = (track - 1) * NUM_MAX_BYTES_TRACK;
      ::gcr_read_sector((unsigned char*)&(mpGcrData->data[offset]),
                        NUM_MAX_BYTES_TRACK,
                        &(fileData[block * 256]),
                        track, sector);
   }
}


QByteArray D64File::getGcrData()
{
   return QByteArray( (const char*)&(mpGcrData->data[0]),
                      (m40Tracks ? 40 : 35) * NUM_MAX_BYTES_TRACK );
}


bool D64File::setGcrData( const QByteArray &data )
{
   if( data.size() != ((m40Tracks ? 40 : 35) * 8192 ) )
   {
      return false;
   }
   formatGcr();
   ::memcpy( &(mpGcrData->data[0]), data.constData(), data.size() );
   convertFromGcr();
   return true;
}


QByteArray D64File::getD64Data()
{
   return mFileData;
}
