/**
 * src/qchaco/D64Widget.cpp
 * written by Sven Oliver Moll
 * 
 * distributed under the terms of the GNU Lesser General Public License (LGPL)
 * available at http://www.gnu.org/licenses/lgpl.html
 */

/* class declaration */
#include "D64Widget.hpp"

/* system headers */

/* Qt headers */
#include <QtGui>
#include <QSettings>

/* local library headers */

/* local headers */
#include "D64File.hpp"
#include "FileNameEdit.hpp"
#include "HexEdit.hpp"
#include "HistoryList.hpp"


D64Widget::D64Widget( QWidget *parent )
: QWidget( parent )
, mpFileName( new FileNameEdit( this ) )
, mpDrive( new QComboBox( this ) )
, mpId1( new HexEdit( 0xFF, this ) )
, mpId2( new HexEdit( 0xFF, this ) )
, mpUpload( new QPushButton( tr("Upload"),this ) )
, mpDownload( new QPushButton( tr("Download"),this ) )
{
   setAcceptDrops( true );
   for( int drive = 1; drive <= 2; ++drive )
   {
      for( int disk = 1; disk <= 4; ++disk )
      {
         mpDrive->addItem( tr("Drive %1 Disk %2").arg(drive).arg(disk) );
      }
   }

   QGridLayout *layout = new QGridLayout( this );
   layout->setContentsMargins( 3, 3, 3, 3 );
   layout->setSpacing( 1 );

   layout->addWidget( new QLabel( tr("File Name:"), this ), 0, 0 );
   layout->addWidget( new QLabel( tr("Drive:"), this ),     0, 1 );
   layout->addWidget( new QLabel( tr("ID1:"), this ),       0, 2 );
   layout->addWidget( new QLabel( tr("ID2:"), this ),       0, 3 );
   layout->addWidget( mpDownload,                           0, 4 );

   layout->addWidget( mpFileName, 1, 0 );
   layout->addWidget( mpDrive,    1, 1 );
   layout->addWidget( mpId1,      1, 2 );
   layout->addWidget( mpId2,      1, 3 );
   layout->addWidget( mpUpload,   1, 4 );

   layout->setRowStretch( 2, 1 );

   setLayout( layout );

   connect( mpUpload, SIGNAL(clicked()),
            this, SLOT(upload()) );
   connect( mpDownload, SIGNAL(clicked()),
            this, SLOT(download()) );
   connect( mpFileName, SIGNAL(fileNameChanged(QString)),
            this, SLOT(loadId(QString)) );
   mpFileName->setRegistryKey( "Floppy" );
}


D64Widget::~D64Widget()
{
}


void D64Widget::deactivate( bool deactivated )
{
   mpDownload->setDisabled( deactivated );
   mpUpload->setDisabled( deactivated );
}


void D64Widget::dragEnterEvent( QDragEnterEvent *event )
{
   if( event->mimeData()->hasFormat( "text/uri-list" ) )
   {
      event->acceptProposedAction();
   }
}


void D64Widget::dropEvent( QDropEvent *event )
{
   QList<QUrl> urls( event->mimeData()->urls() );
   if( urls.isEmpty() )
   {
      return;
   }
   QString fileName( urls.first().toLocalFile() );
   if( fileName.isEmpty() )
   {
      return;
   }
   mpFileName->setFileName( fileName );
}


void D64Widget::loadId( const QString &fileName )
{
   QFile file( fileName );
   file.open( QIODevice::ReadOnly );
   if( file.seek( 0x000165A2 ) ) // hack: position of ID
   {
      mpId1->setValue( file.read(1).at(0) );
      mpId2->setValue( file.read(1).at(0) );
   }
}


unsigned long D64Widget::slotToAddr()
{
   int slot = mpDrive->currentIndex();
   unsigned long baseAddr = 0x00300000L;

   if( (slot >= 0) && (slot <= 3) )
   {
      return baseAddr + (slot * 0x50000L);
   }

   baseAddr = 0x00480000L;
   slot -= 4;

   if( (slot >= 0) && (slot <= 3) )
   {
      return baseAddr + (slot * 0x50000L);
   }

   return baseAddr;
}


void D64Widget::upload()
{
   D64File d64file( mpFileName->fileName() );
   d64file.formatGcr( mpId1->value(), mpId2->value() );
   d64file.convertToGcr();
   mpFileName->addCurrentEntry();
   if( d64file.isValid() )
   {
      emit requestTransferStart();
      emit requestUpload( slotToAddr(), d64file.getGcrData() );
   }
}


void D64Widget::download()
{
   unsigned long size = 0x2000 * 35;
   emit requestTransferStart();
   emit requestDownload( slotToAddr(), size, this, "saveData" );
}


void D64Widget::saveData( const QByteArray &data )
{
   QString saveFileName;
   {
      QSettings settings;
      QString dir( settings.value("SaveD64").toString() );
      saveFileName = QFileDialog::getSaveFileName( this,
                                                   tr("%1: save D64 file").arg( QApplication::applicationName() ),
                                                   dir, "Floppy Image (*.D64)" );
      settings.setValue( "SaveD64", QFileInfo( saveFileName ).absolutePath() );
   }
   if( !saveFileName.isEmpty() )
   {
      D64File d64( 35 );
      d64.setGcrData( data );
      d64.convertFromGcr();

      QFile d64File( saveFileName );
      d64File.open( QIODevice::WriteOnly );
      d64File.write( d64.getD64Data() );
      d64File.close();
   }
}
