/**
 * src/qchaco/FileNameEdit.cpp
 * written by Sven Oliver Moll
 * 
 * distributed under the terms of the GNU Lesser General Public License (LGPL)
 * available at http://www.gnu.org/licenses/lgpl.html
 */

/* class declaration */
#include "FileNameEdit.hpp"

/* system headers */

/* Qt headers */
#include <QComboBox>
#include <QCompleter>
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QDirModel>
#include <QFileDialog>
#include <QLineEdit>
#include <QList>
#include <QPushButton>
#include <QSettings>
#include <QUrl>
#include <QVBoxLayout>

/* local library headers */

/* local headers */

/* class variables */
int FileNameEdit::cMaxEntries = 50;


FileNameEdit::FileNameEdit( QWidget *parent )
: QFrame( parent )
, mpEdit( new QComboBox( this ) )
, mpPushButton( new QPushButton( "...", this ) )
{
   setLineWidth( 1 );
   setFrameShape( QFrame::Panel );
   setFrameShadow( QFrame::Raised );
   mpEdit->setEditable( true );
   mpEdit->setAcceptDrops( false );
   mpPushButton->setMaximumWidth( mpPushButton->height() );
   QBoxLayout *layout = new QHBoxLayout( this );
   layout->setContentsMargins( 1, 0, 1, 0 );
   layout->setSpacing( 1 );
   layout->addWidget( mpEdit, 1 );
   layout->addWidget( mpPushButton );

   connect( mpPushButton, SIGNAL(clicked()),
            this, SLOT(requester()) );
   connect( mpEdit, SIGNAL(textChanged(QString)),
            this, SLOT(checkFileName()) );

   QCompleter *completer = new QCompleter( mpEdit );
   completer->setModel( new QDirModel( mpEdit ) );
   mpEdit->setCompleter( completer );
   setAcceptDrops( true );
}


FileNameEdit::~FileNameEdit()
{
}


void FileNameEdit::requester()
{
   QFileDialog fileDialog( this );

   fileDialog.setFileMode( QFileDialog::ExistingFile );
   QFileInfo fi( mpEdit->lineEdit()->text() );
   fileDialog.setDirectory( fi.absoluteDir() );
   fileDialog.setReadOnly( true );

   if( fileDialog.exec() )
   {
      QString result( fileDialog.selectedFiles().at(0) );
      mpEdit->lineEdit()->setText( result );
   }
}


void FileNameEdit::dragEnterEvent( QDragEnterEvent *event )
{
   if( event->mimeData()->hasFormat( "text/uri-list" ) )
   {
      event->acceptProposedAction();
   }
}


void FileNameEdit::dropEvent( QDropEvent *event )
{
   QList<QUrl> urls = event->mimeData()->urls();
   if( urls.isEmpty() )
   {
      return;
   }
   QString fileName = urls.first().toLocalFile();
   if( fileName.isEmpty() )
   {
      return;
   }
   mpEdit->lineEdit()->setText( fileName );
}


void FileNameEdit::setFileName( const QString &name )
{
   mpEdit->lineEdit()->setText( name );
}


QString FileNameEdit::fileName() const
{
   return mpEdit->lineEdit()->text();
}


void FileNameEdit::checkFileName()
{
   QFileInfo fileInfo( mpEdit->lineEdit()->text() );
   if( fileInfo.isFile() )
   {
      emit fileNameChanged( fileInfo.absoluteFilePath() );
   }
}


QString FileNameEdit::registryKey() const
{
   return mRegistryKey;
}


void FileNameEdit::setRegistryKey( const QString &key )
{
   QSettings settings;
   settings.beginGroup( "History" );
   cMaxEntries = settings.value( "_MAX_ENTRIES_", 50 ).toInt();
   writeToRegistry();
   mRegistryKey = key;
   readFromRegistry();
}


void FileNameEdit::addCurrentEntry()
{
   QString entry( mpEdit->lineEdit()->text() );
   if( entry.isEmpty() )
   {
      return;
   }

   int index = mpEdit->findText( entry );
   if( index >= 0 )
   {
      mpEdit->removeItem( index );
   }
   mpEdit->insertItem( 0, entry );
   while( mpEdit->count() > cMaxEntries )
   {
      mpEdit->removeItem( mpEdit->count() - 1 );
   }

   writeToRegistry();
   mpEdit->setCurrentIndex( 0 );
}


void FileNameEdit::readFromRegistry()
{
   mpEdit->clear();
   if( !mRegistryKey.isEmpty() )
   {
      QSettings settings;
      settings.beginGroup( "History" );
      mpEdit->addItems( settings.value( mRegistryKey ).toStringList() );
      while( mpEdit->count() > cMaxEntries )
      {
         mpEdit->removeItem( mpEdit->count() - 1 );
      }
   }
}


void FileNameEdit::writeToRegistry()
{
   if( !mRegistryKey.isEmpty() )
   {
      QSettings settings;
      settings.beginGroup( "History" );
      QStringList entries;
      for( int i = 0; i < mpEdit->count(); ++i )
      {
         entries.append( mpEdit->itemText( i ) );
      }
      settings.setValue( mRegistryKey, entries );
   }
}


void FileNameEdit::setHistorySize( int size )
{
   cMaxEntries = size;
}
