/**
 * src/qchaco/C64Widget.cpp
 * written by Sven Oliver Moll
 * 
 * distributed under the terms of the GNU Lesser General Public License (LGPL)
 * available at http://www.gnu.org/licenses/lgpl.html
 */

/* class declaration */
#include "FlashWidget.hpp"

/* system headers */

/* Qt headers */
#include <QtGui>
#include <QSettings>

/* local library headers */

/* local headers */
#include "C64File.hpp"
#include "FileNameEdit.hpp"
#include "FlashAddonWidget.hpp"


FlashWidget::FlashWidget( QWidget *parent )
: QWidget( parent )
, mpCoreFileName( new FileNameEdit( this ) )
, mpRomFileName( new FileNameEdit( this ) )
, mpCore( new QComboBox( this ) )
, mpUpload( new QPushButton( tr("Flash"),this ) )
, mpAddAddon( new QPushButton( tr("Add Additional ROM"),this ) )
, mpStartSlot( new QPushButton( tr("Start Slot"),this ) )
, mpAddonsLayout( new QVBoxLayout() )
, mAdditionalRoms()
{
   QSettings settings;
   mpCoreFileName->setRegistryKey( "Flash/Core" );
   mpRomFileName->setRegistryKey( "Flash/Rom");
   settings.beginGroup( "AdditionalRoms" );
   QStringList keys( settings.childKeys() );
   QString value;
   foreach( const QString &key, keys )
   {
      FlashAddonWidget *addon = new FlashAddonWidget( this );
      value = settings.value( key ).toString();
      addon->setTypeByName( value.left(value.indexOf('/')) );
      addon->setFileName( value.mid(value.indexOf('/')) );
      mAdditionalRoms.append( addon );
      connect( addon, SIGNAL(requestRemoval(FlashAddonWidget*)),
               this, SLOT(removeAddon(FlashAddonWidget*)) );
      connect( addon, SIGNAL(requestUploadToRam(FlashAddonWidget*)),
               this, SLOT(uploadToRam(FlashAddonWidget*)) );
   }
   settings.endGroup();

   foreach( FlashAddonWidget *addon, mAdditionalRoms )
   {
      mpAddonsLayout->addWidget( addon );
   }
   for( int i = 0; i < 16; i++ )
   {
      mpCore->addItem( QString::number( i, 16 ) );
   }
   mpCore->setCurrentIndex( settings.value( "Core", 2 ).toInt() );

   QGridLayout *layout = new QGridLayout( this );
   layout->setContentsMargins( 3, 3, 3, 3 );
   layout->setSpacing( 1 );

   layout->addWidget( new QLabel( tr("Core File Name:"), this ), 0, 0 );
   layout->addWidget( mpCoreFileName,   0, 1 );
   layout->addWidget( new QLabel( tr("ID:"),  this ), 0, 2 );
   layout->addWidget( mpCore,           0, 3 );

   layout->addWidget( new QLabel( tr("ROM File Name:"),  this ), 1, 0 );
   layout->addWidget( mpRomFileName,    1, 1 );
   layout->addWidget( mpUpload,         1, 2, 1, 2 );

   layout->addWidget( mpAddAddon,       2, 0, 1, 2 );
   layout->addWidget( mpStartSlot,      2, 2, 1, 2 );
   layout->addLayout( mpAddonsLayout,   3, 0, 1, 4 );

   layout->setRowStretch( 4, 1 );

   setLayout( layout );

   connect( mpUpload, SIGNAL(clicked()),
            this, SLOT(upload()) );
   connect( mpAddAddon, SIGNAL(clicked()),
            this, SLOT(addAddon()) );
   connect( mpStartSlot, SIGNAL(clicked()),
            this, SLOT(start()) );
}


FlashWidget::~FlashWidget()
{
}


void FlashWidget::deactivate( bool deactivated )
{
   mpUpload->setDisabled( deactivated );
   mpStartSlot->setDisabled( deactivated );
   foreach( FlashAddonWidget *addon, mAdditionalRoms )
   {
      addon->deactivate( deactivated );
   }
}


void FlashWidget::uploadToRam( FlashAddonWidget *addon )
{
   unsigned long fileSize = QFileInfo( addon->fileName() ).size();
   bool ok = true;

   emit requestTransferStart();
   switch( addon->type() )
   {
   case FlashAddonWidget::Basic:
      if( fileSize == 0x2000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressBasic, addon->fileName() );
      }
      else
      {
         log( tr("Basic has wrong size: $%1 instead of $2000")
              .arg( QString::number( fileSize, 16 ) ) );
         ok = false;
      }
      break;
   case FlashAddonWidget::Kernal:
      if( fileSize == 0x2000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressKernal, addon->fileName() );
      }
      else
      {
         log( tr("Kernal has wrong size: $%1 instead of $2000")
              .arg( QString::number( fileSize, 16 ) ) );
         ok = false;
      }
      break;
   case FlashAddonWidget::Charset:
      if( fileSize == 0x1000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressCharset, addon->fileName() );
      }
      else
      {
         log( tr("Charset has wrong size: $%1 instead of $1000")
              .arg( QString::number( fileSize, 16 ) ) );
         ok = false;
      }
      break;
   case FlashAddonWidget::FC3:
      if( fileSize == 0x10000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressFinal3, addon->fileName() );
      }
      else
      {
         log( tr("FC3 has wrong size: $%1 instead of $10000")
              .arg( QString::number( fileSize, 16 ) ) );
         ok = false;
      }
      break;
   case FlashAddonWidget::Replay:
      if( fileSize == 0x10000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressReplay, addon->fileName() );
         log( tr("Retro Replay replaced with %1")
              .arg( addon->fileName() ) );
      }
      else if( fileSize == 0x8000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressReplay,          addon->fileName() );
         emit requestUpload( BaseOffsetModule + BaseAddressReplay + 0x8000, addon->fileName() );
         log( tr("Action Replay replaced with %1")
              .arg( addon->fileName() ) );
      }
      else
      {
         log( tr("Replay has wrong size: $%1 instead of $8000 or $10000")
              .arg( QString::number( fileSize, 16 ) ) );
         ok = false;
      }
      break;
   case FlashAddonWidget::Floppy:
      if( fileSize == 0x8000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy1, addon->fileName() );
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy2, addon->fileName() );
      }
      else if( fileSize == 0x4000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy1,          addon->fileName() );
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy1 + 0x4000, addon->fileName() );
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy2,          addon->fileName() );
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy2 + 0x4000, addon->fileName() );
      }
      else if( fileSize == 0x2000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy1 + 0x2000, addon->fileName() );
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy1 + 0x6000, addon->fileName() );
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy2 + 0x2000, addon->fileName() );
         emit requestUpload( BaseOffsetModule + BaseAddressFloppy2 + 0x6000, addon->fileName() );
      }
      else
      {
         log( tr("Floppy has wrong size: $%1 instead of $2000 or $4000")
              .arg( QString::number( fileSize, 16 ) ) );
         ok = false;
      }
      break;
   case FlashAddonWidget::MMC64:
      if( fileSize == 0x2000 )
      {
         emit requestUpload( BaseOffsetModule + BaseAddressMMC64, addon->fileName() );
      }
      else
      {
         log( tr("MMC has wrong size: $%1 instead of $2000")
              .arg( QString::number( fileSize, 16 ) ) );
         ok = false;
      }
      break;
   default:
      break;
   }

   if( ok )
   {
      updateSettings();
      addon->addCurrentEntry();
   }
}


void FlashWidget::replaceLog( unsigned long addr, unsigned long size,
                              const QString &type, const QString &fileName )
{
   log( tr("%1-%2 replaced %3 with %4")
        .arg( QString::number( addr, 16 ).rightJustified( 5, '0' ),
              QString::number( addr + size - 1, 16 ).rightJustified( 5, '0' ),
              type, fileName ) );
}


void FlashWidget::upload()
{
   bool ok = true;
   QByteArray romData;

   if( !mpRomFileName->fileName().isEmpty() )
   {
      QFile romFile( mpRomFileName->fileName() );
      if( !romFile.open( QIODevice::ReadOnly ) )
      {
         log( tr("Can't open rom file %1").arg(romFile.fileName()) );
         return;
      }
      romData = romFile.readAll();
      romFile.close();

      QByteArray tmpData;
      QFile tmpFile;
      foreach( FlashAddonWidget *addon, mAdditionalRoms )
      {
         tmpFile.setFileName( addon->fileName() );
         if( !tmpFile.open( QIODevice::ReadOnly ) )
         {
            continue;
         }
         tmpData = tmpFile.readAll();
         tmpFile.close();
         addon->addCurrentEntry();

         switch( addon->type() )
         {
         case FlashAddonWidget::Basic:
            if( tmpData.size() == 0x2000 )
            {
               romData.replace( BaseAddressBasic, tmpData.size(), tmpData );
               replaceLog( BaseAddressBasic, tmpData.size(), "Basic", addon->fileName() );
            }
            else
            {
               log( tr("Basic has wrong size: $%1 instead of $2000")
                    .arg( QString::number( tmpData.size(), 16 ) ) );
               ok = false;
            }
            break;
      case FlashAddonWidget::Kernal:
            if( tmpData.size() == 0x2000 )
            {
               romData.replace( BaseAddressKernal, tmpData.size(), tmpData );
               replaceLog( BaseAddressKernal, tmpData.size(), "Kernal", addon->fileName() );
            }
            else
            {
               log( tr("Kernal has wrong size: $%1 instead of $2000")
                    .arg( QString::number( tmpData.size(), 16 ) ) );
               ok = false;
            }
            break;
      case FlashAddonWidget::Charset:
            if( tmpData.size() == 0x1000 )
            {
               romData.replace( BaseAddressCharset, tmpData.size(), tmpData );
               replaceLog( BaseAddressCharset, tmpData.size(), "Charset", addon->fileName() );
            }
            else
            {
               log( tr("Charset has wrong size: $%1 instead of $1000")
                    .arg( QString::number( tmpData.size(), 16 ) ) );
               ok = false;
            }
            break;
      case FlashAddonWidget::FC3:
            if( tmpData.size() == 0x10000 )
            {
               romData.replace( BaseAddressFinal3, tmpData.size(), tmpData );
               replaceLog( BaseAddressFinal3, tmpData.size(), "Final Cartridge 3", addon->fileName() );
            }
            else
            {
               log( tr("FC3 has wrong size: $%1 instead of $10000")
                    .arg( QString::number( tmpData.size(), 16 ) ) );
               ok = false;
            }
            break;
      case FlashAddonWidget::Replay:
            if( tmpData.size() == 0x10000 )
            {
               romData.replace( BaseAddressReplay, tmpData.size(), tmpData );
               replaceLog( BaseAddressReplay, tmpData.size(), "Retro Replay", addon->fileName() );
            }
            else if( tmpData.size() == 0x8000 )
            {
               romData.replace( BaseAddressReplay, tmpData.size(), tmpData );
               replaceLog( BaseAddressReplay, tmpData.size(), "Action Replay", addon->fileName() );
               romData.replace( BaseAddressReplay + 0x8000, tmpData.size(), tmpData );
               replaceLog( BaseAddressReplay + 0x8000, tmpData.size(), "Action Replay", addon->fileName() );
            }
            else
            {
               log( tr("Replay has wrong size: $%1 instead of $8000 or $10000")
                    .arg( QString::number( tmpData.size(), 16 ) ) );
               ok = false;
            }
            break;
      case FlashAddonWidget::Floppy:
            if( tmpData.size() == 0x8000 )
            {
               romData.replace( BaseAddressFloppy1, tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy1, tmpData.size(), "extended Floppy 1", addon->fileName() );
               romData.replace( BaseAddressFloppy2, tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy2, tmpData.size(), "extended Floppy 2", addon->fileName() );
            }
            if( tmpData.size() == 0x4000 )
            {
               romData.replace( BaseAddressFloppy1,          tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy1, tmpData.size(), "full Floppy 1 mirror", addon->fileName() );
               romData.replace( BaseAddressFloppy1 + 0x4000, tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy1 + 0x4000, tmpData.size(), "full Floppy 1", addon->fileName() );
               romData.replace( BaseAddressFloppy2,          tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy2, tmpData.size(), "full Floppy 2 mirror", addon->fileName() );
               romData.replace( BaseAddressFloppy2 + 0x4000, tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy2 + 0x4000, tmpData.size(), "full Floppy 2", addon->fileName() );
            }
            else if( tmpData.size() == 0x2000 )
            {
               romData.replace( BaseAddressFloppy1 + 0x2000, tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy1 + 0x2000, tmpData.size(), "upper Floppy 1 mirror", addon->fileName() );
               romData.replace( BaseAddressFloppy1 + 0x6000, tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy1 + 0x6000, tmpData.size(), "upper Floppy 1", addon->fileName() );
               romData.replace( BaseAddressFloppy2 + 0x2000, tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy2 + 0x2000, tmpData.size(), "upper Floppy 2 mirror", addon->fileName() );
               romData.replace( BaseAddressFloppy2 + 0x6000, tmpData.size(), tmpData );
               replaceLog( BaseAddressFloppy2 + 0x6000, tmpData.size(), "upper Floppy 2", addon->fileName() );
            }
            else
            {
               log( tr("Floppy has wrong size: $%1 instead of $2000, $4000 or $8000")
                    .arg( QString::number( tmpData.size(), 16 ) ) );
               ok = false;
            }
            break;
      case FlashAddonWidget::MMC64:
            if( tmpData.size() == 0x2000 )
            {
               romData.replace( BaseAddressMMC64, tmpData.size(), tmpData );
               replaceLog( BaseAddressMMC64, tmpData.size(), "MMC64", addon->fileName() );
            }
            else
            {
               log( tr("MMC64 has wrong size: $%1 instead of $2000")
                    .arg( QString::number( tmpData.size(), 16 ) ) );
               ok = false;
            }
            break;
      default:
            break;
         }
      }
   }

   if( ok )
   {
      mpCoreFileName->addCurrentEntry();
      mpRomFileName->addCurrentEntry();
      emit requestTransferStart();
      emit requestFlash( mpCore->currentIndex(),
                         mpCoreFileName->fileName(),
                         romData );
      emit requestStart( mpCore->currentIndex() );
      updateSettings();
   }
}


void FlashWidget::updateSettings()
{
   QSettings settings;
   settings.setValue( "Core", mpCore->currentIndex() );
   settings.remove( "AdditionalRoms" );
   settings.beginGroup( "AdditionalRoms" );
   int i = 0;
   foreach( FlashAddonWidget *addon, mAdditionalRoms )
   {
      settings.setValue( QString::number(i++).rightJustified(4,'0'),
                         addon->typeName() + addon->fileName() );
   }
}


void FlashWidget::addAddon()
{
   FlashAddonWidget *addon = new FlashAddonWidget( this );
   addon->deactivate( !mpUpload->isEnabled() );
   mpAddonsLayout->addWidget( addon );
   mAdditionalRoms.append( addon );
   connect( addon, SIGNAL(requestRemoval(FlashAddonWidget*)),
            this, SLOT(removeAddon(FlashAddonWidget*)) );
   connect( addon, SIGNAL(requestUploadToRam(FlashAddonWidget*)),
            this, SLOT(uploadToRam(FlashAddonWidget*)) );
}


void FlashWidget::removeAddon( FlashAddonWidget* addon )
{
   mpAddonsLayout->removeWidget( addon );
   mAdditionalRoms.removeOne( addon );
   delete addon;
}


void FlashWidget::start()
{
   emit requestTransferStart();
   emit requestStart( mpCore->currentIndex() );
}
