/**
 * src/qchaco/MainWidget.cpp
 * written by Sven Oliver Moll
 *
 * distributed under the terms of the GNU Lesser General Public License (LGPL)
 * available at http://www.gnu.org/licenses/lgpl.html
 */

/* class declaration */
#include "MainWidget.hpp"

/* system headers */

/* Qt headers */
#include <QtGui>
#include <QHostInfo>
#include <QUdpSocket>
#include <QTextCursor>

/* local library headers */

/* local headers */
#include "Version.hpp"
#include "ChacoCmd.hpp"
#include "ConfigWidget.hpp"
#include "FileNameEdit.hpp"
#include "PrgWidget.hpp"
#include "D64Widget.hpp"
#include "FlashWidget.hpp"


MainWidget::MainWidget( int tab, QWidget *parent )
: QSplitter( Qt::Vertical, parent )
, mpTabs( new QTabWidget( this ) )
, mpLog( new QListWidget( this ) )
, mpChacoCmd( new ChacoCmd( 0 ) )
, mpConfigWidget( new ConfigWidget( mpChacoCmd, this ) )
, mpFlashWidget( new FlashWidget( this ) )
, mpPrgWidget( new PrgWidget( this ) )
, mpD64Widget( new D64Widget( this ) )
{
   setContentsMargins( 3, 3, 3, 3 );
   mpLog->addItem( QString("%1 Version %2").arg( QApplication::applicationName(), QCHACO_VERSION ) );
   mpLog->addItem( tr("Intended for use with Chameleon Version %1").arg( CHAMELEON_VERSION ) );
   mpLog->setSelectionMode( QAbstractItemView::ContiguousSelection );

   addWidget( mpTabs );
   mpTabs->addTab( mpConfigWidget, tr("Settings") );
   mpTabs->addTab( mpFlashWidget,  tr("Flash") );
   mpTabs->addTab( mpPrgWidget,    tr("Memory") );
   mpTabs->addTab( mpD64Widget,    tr("Floppy") );
   if( (tab > 0) && (tab < mpTabs->count()) )
   {
      mpTabs->setCurrentIndex( tab );
   }
   addWidget( mpLog );
   QPoint regSizes( QSettings().value( "SplitterSizes", QPoint(1,1) ).toPoint() );
   QList<int> sizes;
   sizes << regSizes.x() << regSizes.y();
   setSizes( sizes );

   connect( mpChacoCmd, SIGNAL(error(int)),
            this, SLOT(handleError(int)) );
   connect( mpChacoCmd, SIGNAL(output(const QString&)),
            this, SLOT(log(const QString&)) );
   connect( mpPrgWidget, SIGNAL(requestTransferStart()),
            mpChacoCmd, SLOT(resetError()) );
   connect( mpPrgWidget, SIGNAL(requestUpload(ulong,const QByteArray&)),
            mpChacoCmd, SLOT(upload(ulong,const QByteArray&)) );
   connect( mpPrgWidget, SIGNAL(requestUpload(ulong,const QString&)),
            mpChacoCmd, SLOT(upload(ulong,const QString&)) );
   connect( mpD64Widget, SIGNAL(requestTransferStart()),
            mpChacoCmd, SLOT(resetError()) );
   connect( mpD64Widget, SIGNAL(requestUpload(ulong,const QByteArray&)),
            mpChacoCmd, SLOT(upload(ulong,const QByteArray&)) );
   connect( mpD64Widget, SIGNAL(requestDownload(ulong,ulong,QObject*,const QString&)),
            mpChacoCmd, SLOT(download(ulong,ulong,QObject*,QString)) );
   connect( mpFlashWidget, SIGNAL(requestTransferStart()),
            mpChacoCmd, SLOT(resetError()) );
   connect( mpFlashWidget, SIGNAL(requestUpload(ulong,const QByteArray&)),
            mpChacoCmd, SLOT(upload(ulong,const QByteArray&)) );
   connect( mpFlashWidget, SIGNAL(requestUpload(ulong,const QString&)),
            mpChacoCmd, SLOT(upload(ulong,const QString&)) );
   connect( mpFlashWidget, SIGNAL(requestStart(int)),
            mpChacoCmd, SLOT(startCore(int)) );
   connect( mpFlashWidget, SIGNAL(requestFlash(int,QString,QByteArray)),
            mpChacoCmd, SLOT(flashRbf(int,QString,QByteArray)) );
   connect( mpFlashWidget, SIGNAL(log(QString)),
            this, SLOT(log(QString)) );
   connect( mpChacoCmd, SIGNAL(active(bool)),
            mpFlashWidget, SLOT(deactivate(bool)) );
   connect( mpChacoCmd, SIGNAL(active(bool)),
            mpPrgWidget, SLOT(deactivate(bool)) );
   connect( mpChacoCmd, SIGNAL(active(bool)),
            mpD64Widget, SLOT(deactivate(bool)) );
}


MainWidget::~MainWidget()
{
   QList<int> regSizes = sizes();
   if( regSizes.count() > 1 )
   {
      QSettings().setValue( "SplitterSizes", QPoint(regSizes.at(0),regSizes.at(1)) );
   }
   mpChacoCmd->quit();
   if( !mpChacoCmd->wait( 1000 ) )
   {
      mpChacoCmd->terminate();
      mpChacoCmd->wait( 500 );
   }
}


void MainWidget::handleError( int retval )
{
   QString message( tr("An error occured running %1:\n%2") );
   QString detail;
   if( retval & 0x20000000 )
   {
      detail = tr("The process could not be started.\n"
                  "Does the file exist and is it executeable?");
   }
   else if( retval & 0x10000000 )
   {
      detail = tr("The process crashed. This was not supposed to happen.");
   }
   else
   {
      detail = tr("The error code %1 was returned.").arg( retval );
   }

   QMessageBox::critical( this, QApplication::applicationName(),
                          message.arg( mpChacoCmd->filePath(), detail ) );
}


void MainWidget::log( const QString &text )
{
   QListWidgetItem *item;
   int last = mpLog->count() - 1;
   if( text.startsWith("\r") )
   {
      item = mpLog->takeItem( last );
      if( item )
      {
         delete item;
      }
      mpLog->addItem( text.mid(1) );
   }
   else
   {
      mpLog->addItem( text );

      while( mpLog->count() > 1024 )
      {
         item = mpLog->takeItem(0);
         if( item )
         {
            delete item;
         }
      }

      mpLog->scrollToBottom();
   }
}


int MainWidget::currentTab()
{
   return mpTabs->currentIndex();
}
