/**
 * src/qchaco/PrgWidget.cpp
 * written by Sven Oliver Moll
 * 
 * distributed under the terms of the GNU Lesser General Public License (LGPL)
 * available at http://www.gnu.org/licenses/lgpl.html
 */

/* class declaration */
#include "PrgWidget.hpp"

/* system headers */

/* Qt headers */
#include <QtGui>
#include <QSettings>

/* local library headers */

/* local headers */
#include "C64File.hpp"
#include "FileNameEdit.hpp"
#include "HexEdit.hpp"


PrgWidget::PrgWidget( QWidget *parent )
: QWidget( parent )
, mpFileName( new FileNameEdit( this ) )
, mpAutostart( new QCheckBox( tr("Autostart"), this ) )
, mpTakeLoadAddress( new QCheckBox( tr("Use first two bytes as load address"), this ) )
, mpUpload( new QPushButton( tr("Upload"),this ) )
, mpStartAddress( new HexEdit( 0x1FFFFFF, this ) )
, mpEndAddress( new HexEdit( 0x1FFFFFF, this ) )
, mDataSize( 0 )
{
   mpFileName->setRegistryKey( "Memory" );
   mpAutostart->setChecked( true );
   mpTakeLoadAddress->setChecked( true );
   mpEndAddress->setReadOnly( true );
   QPalette pal( mpEndAddress->palette() );
   pal.setColor( QPalette::Base, QApplication::palette().color( QPalette::Window ) );
   mpEndAddress->setPalette( pal );

   setAcceptDrops( true );

   QGridLayout *layout = new QGridLayout( this );
   layout->setContentsMargins( 3, 3, 3, 3 );
   layout->setSpacing( 1 );

   layout->addWidget( new QLabel( tr("File Name:"), this ), 0, 0 );

   layout->addWidget( mpFileName,   1, 0, 1, 3 );
   layout->addWidget( mpUpload,     1, 3 );

   layout->addWidget( mpAutostart,                              2, 0 );
   layout->addWidget( mpTakeLoadAddress,                        2, 1, 1, 3 );
   layout->addWidget( new QLabel( tr("Start Address:"), this ), 3, 0 );
   layout->addWidget( mpStartAddress,                           3, 1 );
   layout->addWidget( new QLabel( tr("End Address:"), this ),   4, 0 );
   layout->addWidget( mpEndAddress,                             4, 1 );
   layout->setColumnStretch( 2, 1 );
   layout->setRowStretch( 5, 1 );

   setLayout( layout );

   connect( mpUpload, SIGNAL(clicked()),
            this, SLOT(upload()) );
   connect( mpStartAddress, SIGNAL(valueChanged(ulong)),
            this, SLOT(updateConfig()) );
   connect( mpFileName, SIGNAL(fileNameChanged(QString)),
            this, SLOT(analyse(QString)) );
   connect( mpTakeLoadAddress, SIGNAL(clicked()),
            this, SLOT(updateConfig()) );
   analyse( mpFileName->fileName() );
}


PrgWidget::~PrgWidget()
{
}


void PrgWidget::deactivate( bool deactivated )
{
   mpUpload->setDisabled( deactivated );
}


void PrgWidget::analyse( const QString &fileName )
{
   if( mpTakeLoadAddress->isChecked() )
   {
      QFile file( fileName );
      if( !file.open( QIODevice::ReadOnly ) )
      {
         return;
      }
      mDataSize = file.size();
      if( mDataSize > 2 )
      {
         QByteArray data( file.read( 2 ) );
         unsigned long addr = ((unsigned char)data.at(1) << 8) |
                              (unsigned char)data.at(0);
         if( addr + file.size() <= 0xffff )
         {
            mpStartAddress->setValue( addr );
            mpEndAddress->setValue( addr + mDataSize - 1 );
         }
      }
      file.close();
   }
   updateConfig();
}


void PrgWidget::updateConfig()
{
   QFileInfo fileInfo( mpFileName->fileName() );
   mpTakeLoadAddress->setEnabled( false );
   mpAutostart->setEnabled( false );
   if( fileInfo.size() < 0x10000 )
   {
      if( mpStartAddress->value() + fileInfo.size() < 0x10000 )
      {
         mpTakeLoadAddress->setEnabled( true );
         if( mpTakeLoadAddress->isChecked() )
         {
            mpAutostart->setEnabled( true );
         }
      }
   }
   mpEndAddress->setValue( mpStartAddress->value() + fileInfo.size() - 1 );
}


void PrgWidget::dragEnterEvent( QDragEnterEvent *event )
{
   if( event->mimeData()->hasFormat( "text/uri-list" ) )
   {
      event->acceptProposedAction();
   }
}


void PrgWidget::dropEvent( QDropEvent *event )
{
   QList<QUrl> urls( event->mimeData()->urls() );
   if( urls.isEmpty() )
   {
      return;
   }
   QString fileName( urls.first().toLocalFile() );
   if( fileName.isEmpty() )
   {
      return;
   }
   mpFileName->setFileName( fileName );
}


void PrgWidget::upload()
{
   emit requestTransferStart();

   if( mpTakeLoadAddress->isEnabled() && mpTakeLoadAddress->isChecked() )
   {
      QFile file( mpFileName->fileName() );
      QByteArray addr;
      unsigned long uploadAddr;
      unsigned long endAddr;
      if( !file.open( QIODevice::ReadOnly ) )
      {
         return;
      }
      mpFileName->addCurrentEntry();
      addr = file.read( 2 );
      uploadAddr = (unsigned char)addr.at( 0 ) |
                   ((unsigned char)addr.at( 1 ) << 8);
      emit requestUpload( uploadAddr, file.readAll() );
      endAddr = uploadAddr + file.size() - 1;
      file.close();

      addr[0] = endAddr & 0xFF;
      addr[1] = (endAddr >> 8) & 0xFF;
      emit requestUpload( 174, addr );
      addr = addr + addr + addr;
      emit requestUpload( 45, addr );

      if( mpAutostart->isEnabled() && mpAutostart->isChecked() )
      {
         QByteArray key1Buffer;
         QByteArray key2Buffer;
         if( uploadAddr == 0x0801 )
         {
            key1Buffer = "\x93RUN\r";
         }
         else
         {
            key1Buffer = QString( "\x93SYS%1\r" )
                         .arg( uploadAddr )
                         .toLatin1();
         }
         key2Buffer.append( (char)(key1Buffer.size() & 0xf) );
         emit requestUpload( 631, key1Buffer );
         emit requestUpload( 198, key2Buffer );
      }
   }
   else
   {
      emit requestUpload( mpStartAddress->value(), mpFileName->fileName() );
   }
}
